#!/usr/bin/env python
# -*- coding: utf-8 -*-

from setuptools import setup

setup(name='packagetest108',
      version='0.108',
      description='DevDoo Microservices',
      url='http://github.com/storborg/funniest',
      author='Israel Agoeiro',
      author_email='israelagoeiro@gmail.com',
      license='MIT',
      packages=['devdoo', 'devdoo/yaml'],
      zip_safe=False)
